#!/usr/bin/python

import sys
import re
import os
import pwd

def get_home_dir ():
    entry = pwd.getpwuid (os.getuid ())

    return entry[5]

def check_has_gtkrc ():
    gtkrc = os.path.join (get_home_dir (), ".gtkrc")
    return os.path.exists (gtkrc)

def switch_to_raleigh ():
    os.system ("""
    
if [ -f ~/.gtkrc ] ; then
  /bin/mv ~/.gtkrc ~/.gtkrc.PREUPGRADE && cat ~/.gtkrc.PREUPGRADE | sed -e 's/themes\/[a-zA-Z0-9]*/themes\/Raleigh/g' > ~/.gtkrc
else
  /bin/cp /etc/skel/.gtkrc ~/.gtkrc
fi

               """)

# First, if they never set the theme, switch to Raleigh, so
# we look good
if not check_has_gtkrc ():
    switch_to_raleigh ()

import gtk
import gnome
import gnome.ui
import gnome.config
import libglade
import signal
import GdkImlib
from gtk import TRUE
from gtk import FALSE
try:
    import cString
    string = cString
except:
    import string
import errno

debug_mode = FALSE

# Name of the current theme
theme_name = None

# Whether we have to upgrade because the theme no longer exists
theme_switch_forced = FALSE

# Whether we need to ask the user if we need to switch the theme
check_theme_switch = FALSE

# The user's selections
apply_wm = FALSE
apply_panel = FALSE
apply_nautilus = FALSE
apply_theme = FALSE

version_serial = 140

def mark_already_run ():
    global version_serial

    if not debug_mode:
        gnome.config.set_int ("/gnome-upgrade/Config/runbefore", version_serial)
        gnome.config.sync ()
    else:
        print "not marking already run, in debug mode"

# if there is no ~/.gnome/panel file, then chances are, they've never run gnome, and we can exit
try:
    os.stat (get_home_dir() + "/.gnome/panel.d")
except:
    if debug_mode:
        print "never run gnome, only pull in gtkrc"
    mark_already_run ()
    sys.exit ()

if not debug_mode:
    # check the version
    runbefore = gnome.config.get_int ("/gnome-upgrade/Config/runbefore=0")
    if debug_mode:
            print "runbefore = %d" % runbefore
    if runbefore >= version_serial:
        sys.exit()

class WMConfig:
    def __init__ (self):
        self.focus_mode = 'click'
        self.move_mode = 'opaque'
        self.resize_mode = 'box'
        self.tooltips = 1
        self.num_desktops = 4
        self.area_rows = 1
        self.area_columns = 1

    def spew (self):
        print "focus: " + self.focus_mode
        print "move: " + self.move_mode
        print "resize: " + self.resize_mode
        print "tooltips: " + ("%d" % self.tooltips)
        print "num_desktops: %d" % self.num_desktops
        print "virtual area rows: %d" % self.area_rows
        print "virtual area columns: %d" % self.area_columns
        

def load_e_config ():
    e_config_line = re.compile ('([0-9]+)(.*)')

    e_config_file = os.path.join (get_home_dir (), ".enlightenment/...e_session-XXXXXX")

    handle = None
    try:
        handle = open (e_config_file)
    except:
        return WMConfig ()

    conf = WMConfig ()

    lines = handle.readlines ()

    for line in lines:
        match = e_config_line.match (line)
        if match:
            magic_crackrock_code = string.strip (match.group (1))
            value = string.strip (match.group (2))

            if magic_crackrock_code == '307':
                if value == '0':
                    conf.focus_mode = 'mouse'
                elif value == '1':
                    conf.focus_mode = 'sloppy'
                elif value == '2':
                    conf.focus_mode = 'click'
                        
            elif magic_crackrock_code == '311':
                if value == '0':
                    conf.move_mode = 'opaque'
                else:
                    conf.move_mode = 'box'
            elif magic_crackrock_code == '312':
                if value == '0':
                    conf.resize_mode = 'opaque'
                else:
                    conf.resize_mode = 'box'
            elif magic_crackrock_code == '17':
                if value == '0':
                    conf.tooltips = 0
            elif magic_crackrock_code == '333':
                conf.num_desktops = string.atoi (value)
            elif magic_crackrock_code == '407':
                (columns, rows) = string.split (value, ' ')
                conf.area_rows = string.atoi (rows)
                conf.area_columns = string.atoi (columns)

    return conf

def save_sawfish_config (conf):
    dir = os.path.join (get_home_dir (), ".sawfish")
    file = os.path.join (dir, "custom")

    try:
        os.mkdir (dir)
    except OSError, e:
        if e.errno != errno.EEXIST:
            return

    ## only save a sawfish config if none existed previously
    if os.path.exists (file):
        return
    
    file = os.path.join (dir, "UPGRADE-custom.jl")

    handle = None
    try:
        # use append mode, just for paranoia
        handle = open (file, 'a')
    except:
        return

    sawfish_focus = 'click'
    if conf.focus_mode == 'mouse':
        sawfish_focus = 'enter-exit'
    elif conf.focus_mode == 'sloppy':
        sawfish_focus = 'enter-only'

    handle.write ("(require 'sawfish.wm.customize)\n")
    handle.write ("(require 'sawfish.wm.commands.move-resize)\n")
    handle.write ("(require 'sawfish.wm.ext.tooltips)\n")
    handle.write ("(require 'sawfish.wm.workspace)\n")
    
    handle.write ("(customize-set (quote focus-mode) (quote %s))\n" % sawfish_focus)

    handle.write ("(customize-set (quote resize-outline-mode) (quote %s))\n" % conf.resize_mode)

    handle.write ("(customize-set (quote move-outline-mode) (quote %s))\n" % conf.move_mode)

    sawfish_tooltips = 't'
    if not conf.tooltips:
        sawfish_tooltips = 'nil'

# Probably they didn't really want tooltips
#    handle.write ("(customize-set (quote tooltips-enabled) (quote %s) (quote tooltips))\n" % sawfish_tooltips)

    # The following is an awful hack to prevent a hook from dividing by zero since
    # we dont' have a display connection
    handle.write ("(set 'workspace-geometry-changed '())\n")
    handle.write ("(customize-set (quote workspace-geometry) '(%s . (%s . %s)))\n" % (conf.num_desktops, conf.area_columns, conf.area_rows))

    handle.write ("(customize-write-user-file)\n")

    handle.close

    handle = None

    os.system ("sawfish --batch %s && rm %s" % (file, file))
    
def set_sawfish_default ():
    section_line = re.compile ('^\[(.*)\]')
    eline = re.compile ('([0-9]+).*enlightenment')
    in_default = FALSE
    e_priority = None

    session_file = os.path.join (get_home_dir (), ".gnome/session")
    session_out_file = os.path.join (get_home_dir (), ".gnome/session.NOE")

    in_handle = None
    out_handle = None
    try:
        in_handle = open (session_file, 'r')
        out_handle = open (session_out_file, 'w')
    except:
        return

    #find the priority in the section.
    lines = in_handle.readlines ()
    for line in lines:
        match = section_line.match (line)
        if match:
            if match.group (1) == "Default":
                in_default = TRUE
            else:
                in_default = FALSE

        if in_default == FALSE:
            continue
        match = eline.match (line)
        if match:
            e_priority = match.group (1)

    if e_priority == None:
        in_handle.close ()
        out_handle.close ()
        return

    written_session = FALSE
    re_string = ("^%s," % e_priority)
    skip_line = re.compile (re_string)
    for line in lines:
        match = section_line.match (line)
        if match:
            if match.group (1) == "Default":
                in_default = TRUE
            else:
                in_default = FALSE

        if not in_default or not skip_line.match (line):
            out_handle.write (line)
        elif (written_session == FALSE):
            written_session = TRUE
            out_handle.write ("""%s,id=default%s
%s,Priority=10
%s,RestartCommand=sawfish --sm-client-id default%s
""" % (e_priority, e_priority, e_priority, e_priority, e_priority))

    in_handle.close ()
    out_handle.close ()
    os.rename (session_out_file, session_file)
    gnome.config.set_string ("/wm-properties/Config/Config/Current", "Sawfish")

def check_gtkrc ():
    global theme_name
    
    import_line = re.compile ('include \"(.*)\"')
    themename_line = re.compile ('include \".*/(.*)/gtk/gtkrc\"')
    gtkrc = os.path.join (get_home_dir (), ".gtkrc")

    handle = None
    try:
        handle = open (gtkrc)
    except:
        return FALSE

    lines = handle.readlines ()

    for line in lines:
        match = import_line.match (line)
        if match:
            match2 = themename_line.match (line)
            if match2:
                theme_name = match2.group (1)
            else:
                theme_name = "Unknown Theme"
            if not os.access (match.group (1), os.R_OK):
                return TRUE
            else:
                return FALSE
    return FALSE

def grep_for_sawmill (filename):
    sawmill_re = re.compile ('sawmill')
    handle = None
    try:
         handle = open (filename)
    except:
        return FALSE

    lines = handle.readlines ()
    for line in lines:
        match = sawmill_re.search (line)
        if match:
            return TRUE

    return FALSE

def check_sawmill ():
    if grep_for_sawmill (os.path.join (get_home_dir (), ".gnome/session")):
        return TRUE

    if grep_for_sawmill (os.path.join (get_home_dir (), ".gnome/wm-properties/Config")):
        return TRUE

    return FALSE
    
def fix_sawmill ():
    session_file = os.path.join (get_home_dir (), ".gnome/session")
    wm_file = os.path.join (get_home_dir (), ".gnome/wm-properties/Config")

    if grep_for_sawmill (session_file):
	command = "/usr/bin/sawmill-replace.sh %s" % session_file
        print "Running: %s" % command
        os.system (command)

    if grep_for_sawmill (wm_file):
        command = "/usr/bin/sawmill-replace.sh %s" % wm_file
        print "Running: %s" % command
        os.system (command)

    os.system ("/bin/cp ~/.sawmillrc ~/.sawfishrc")
    os.system ("/bin/cp -a ~/.sawmill ~/.sawfish")

def check_nautilus ():
    try:
        os.stat ("/usr/bin/nautilus")
    except:
        if debug_mode:
            print "no Nautilus installed, not switching to it"
        return FALSE

    nautilus_re = re.compile ('Program=nautilus')
    
    session_file = os.path.join (get_home_dir (), ".gnome/session")

    contents = open (session_file, 'r').read ()
    if nautilus_re.search (contents):
        return FALSE # already have Nautilus
    else:
        return TRUE
    
def switch_to_nautilus ():
    session_file = os.path.join (get_home_dir (), ".gnome/session")
    session_out_file = os.path.join (get_home_dir (), ".gnome/session.NOGMC")

    in_handle = None
    out_handle = None
    try:
        in_handle = open (session_file, 'r')
        out_handle = open (session_out_file, 'w')
    except:
        return

    gmc_contents = in_handle.read ()

    whitespace_re = re.compile ('\s*')
    session_re = re.compile ('^ *(\[[A-Za-z0-9]+\]) *$\n', re.MULTILINE | re.DOTALL)
    session_name_re = re.compile ('\[([A-Za-z0-9]+)\]')
    gmc_re = re.compile ('([0-9]+),Program=gmc')
    n_clients_re = re.compile ('num_clients=([0-9]+)')

    sessions = session_re.split (gmc_contents)

    new_sessions = []
    for s in sessions:
        foo = whitespace_re.sub ('', s)
        if foo != '':
            new_sessions.append (s)

    sessions = new_sessions

    new_sessions = []
    name = None
    for s in sessions:
        if name:
            new_sessions.append ((name, s))
            name = None
        else:
            match = session_name_re.match (s)
            if match:
                name = match.group (1)

    sessions = new_sessions

    blank_line_re = re.compile ('^$\n', re.MULTILINE)
    nautilus_re = re.compile ('Program=nautilus')

    if debug_mode:
        print '%d sessions' % len (sessions)
    
    for s in sessions:
        name = s[0]
        contents = s[1]

        if debug_mode:
            print "[%s]" % name

        match = gmc_re.search (contents)

        num = -1
        if match:
            ## need to remove gmc entry and replace with Nautilus entry
            num = string.atoi (match.group (1))
            if debug_mode:
                print "gmc is number %d" % num
        else:
            if debug_mode:
                print "no gmc in session"

        match = n_clients_re.search (contents)
        n_clients = -1
        if match:
            n_clients = string.atoi (match.group (1))
            if debug_mode:
                print "%d clients in session" % n_clients
        else:
            if debug_mode:
                print "didn't find number of clients!"
            n_clients = 0

        ## remove num_clients= entry
        contents = n_clients_re.sub ('', contents)

        if num >= 0:
            nautilus_number = num
        else:
            nautilus_number = n_clients

        if not nautilus_re.search (contents):
            nautilus_entry = """### Entry added by gnome-upgrade.py\n%d,id=default%d\n%d,RestartCommand=nautilus start-here: -g 600x410+150+100 --sm-client-id=default%d\n### End entry added by gnome-upgrade.py\n""" % (nautilus_number, nautilus_number, nautilus_number, nautilus_number)
        else:
            nautilus_entry = "" # already have nautilus
        
        if num >= 0:
            entry_re_str = '(^%d,.*\n)+' % num
            entry_re = re.compile (entry_re_str, re.MULTILINE)

            contents = entry_re.sub (nautilus_entry, contents)
            print "Replacing gmc with Nautilus in session %s" % name
        elif nautilus_entry != "":
            contents = contents + nautilus_entry
            n_clients = n_clients + 1
            print "Adding Nautilus to session %s" % name 

        output = '[' + name + ']\n' + contents + ("num_clients=%d\n" % n_clients)
        output = blank_line_re.sub ('', output)
        output = output + '\n'

        out_handle.write (output)
        
    in_handle.close ()
    out_handle.close ()
    if not debug_mode:
        os.rename (session_out_file, session_file)
        os.system ("for I in ~/.gnome-desktop/fd[0-9]; do if test -L $I; then rm -f $I; elif test -d $I; then rmdir $I; fi; done")

# Are we running gnome-1.0, 1.2, or 1.4.  If we are running gnome-1.0, or
# 1.2 this won't have been set.
gnome10 = gnome.config.get_string ("/panel/Config/screenshot_key=UNSET")
if gnome10 == "UNSET":
    panel_needed = TRUE
else:
    panel_needed = FALSE

# Figure out which of the druid pages we need.
wm = gnome.config.get_string ("/wm-properties/Config/Config/Current=UNSET")
if wm == "Enlightenment":
    wm_needed = TRUE
else:
    wm_needed = FALSE

if check_has_gtkrc ():
    check_theme_switch = TRUE

if check_theme_switch:
    theme_switch_forced = check_gtkrc ()
    if theme_switch_forced:
        apply_theme = TRUE
    elif theme_name == "Raleigh":
        check_theme_switch = FALSE

sawmill_needed = check_sawmill ()

nautilus_needed = check_nautilus ()

if debug_mode:
    if not theme_switch_forced:
        theme_switch_forced = 'foobar'

page_list = []

def next_callback (druidpage, druid_broken, page):
    global xml
    
    page.on_next ()

    druid = xml.get_widget ("druid")

    druid.set_page (page_list[page.page_num + 1].get_page ())

    # true means we handled the page change 
    return TRUE        

def back_callback (druidpage, druid_broken, page):
    global xml
    
    druid = xml.get_widget ("druid")
    druid.set_page (page_list[page.page_num - 1].get_page ())

    # true means we handled the page change 
    return TRUE        

def prepare_callback (druidpage, druid, page):
    page.on_prepare ()

def finish_callback (druidpage, druid, page):
    page.on_finish ()


next_page_num = 0

class Page:
    def __init__ (self):
        global next_page_num
        global xml
        
        self.page_num = next_page_num
        next_page_num = next_page_num + 1

        self.druidpage = xml.get_widget (self.page_name)
        if not self.druidpage:
            print "%s not found" % self.page_name
        self.druidpage.connect ("next", next_callback, self)
        self.druidpage.connect ("back", back_callback, self)
        self.druidpage.connect ("finish", finish_callback, self)
        self.druidpage.connect ("prepare", prepare_callback, self)

    def on_prepare (self):
        pass
        
    def on_next (self):
        return TRUE

    def on_finish (self):
        pass

    def get_page (self):
        global xml

        return self.druidpage

class TitlePage (Page):
    def __init__ (self):
        self.page_name = "title_page"
        Page.__init__ (self)

class NautilusPage (Page):
    def __init__ (self):
        global xml
        
        self.page_name = "nautilus_page"
        Page.__init__ (self)
        
        cbutton = xml.get_widget ("nautilus_radio")
        cbutton.set_active (TRUE)

    def on_next (self):
        global apply_nautilus
        global xml
        
        cbutton = xml.get_widget ("nautilus_radio")
        
        if (cbutton.active):
            apply_nautilus = TRUE
        else:
            apply_nautilus = FALSE

        return TRUE

class WmPage (Page):
    def __init__ (self):
        global xml
        
        self.page_name = "wm_page"
        Page.__init__ (self)
        
        cbutton = xml.get_widget ("sawfish_radio")
        cbutton.set_active (TRUE)

    def on_next (self):
        global apply_wm
        global xml
        
        cbutton = xml.get_widget ("sawfish_radio")
        
        if (cbutton.active):
            apply_wm = TRUE
        else:
            apply_wm = FALSE

        return TRUE

class PanelPage (Page):
    def __init__ (self):
        global xml
        
        self.page_name = "panel_page"
        Page.__init__ (self)

        cbutton = xml.get_widget ("panel_radio")
        cbutton.set_active (TRUE)

    def on_next (self):
        global apply_panel
        global xml

        cbutton = xml.get_widget ("panel_radio")
        if not cbutton.active:
            apply_panel = TRUE
        else:
            apply_panel = FALSE        

        return TRUE

class ThemePage (Page):
    def __init__ (self):
        self.page_name = "theme_page"
        Page.__init__ (self)

    def on_prepare (self):
        global xml
        
        label = xml.get_widget ("wm_warning_label")
        label.set_text ("""Your current GTK+ theme, \"%s\" is no longer shipped with the Red Hat
packages.  Your theme will be switched back to the default theme.

Your old theme is likely available at http://gtk.themes.org/""" % theme_name)
        
class ThemeSwitchPage (Page):
    def __init__ (self):
        self.page_name = "theme_switch_page"
        Page.__init__ (self)

        cbutton = xml.get_widget ("theme_radio")
        cbutton.set_active (TRUE)

    def on_next (self):
        global apply_panel
        global xml

        cbutton = xml.get_widget ("theme_radio")
        if cbutton.active:
            apply_theme = TRUE
        else:
            apply_theme = FALSE        

        return TRUE

class FinishPage (Page):
    def __init__ (self):
        self.page_name = "finish_page"
        Page.__init__ (self)

    def on_prepare (self):
        global apply_wm
        global apply_theme
        global apply_panel
        global sawmill_needed
        global apply_nautilus
        global xml
        
        if (not apply_wm) and (not apply_panel) and (not sawmill_needed) and (not apply_nautilus) and (not apply_theme):
            text = """You have elected not to upgrade your GNOME desktop.

Pressing \"Finish\" will continue the log-in process."""
        else:
            text = "Please press \"Finish\" to activate the following changes:\n\n"
        if apply_nautilus:
            text = text + "     Switch file manager to Nautilus\n"
        if apply_wm:
            text = text + "     Switch window manager to sawfish\n"
        if apply_panel:
            text = text + "     Reset panel to defaults\n"
        if apply_theme:
            text = text + "     Switch to new default theme\n"
        if sawmill_needed:
            text = text + "     Replace 'sawmill' with 'sawfish' as your default window manager\n"
            if os.path.exists (os.path.join (get_home_dir (), '.sawmillrc')) and not os.path.exists (os.path.join (get_home_dir (), '.sawfishrc')): 
                text = text + "     Copy '~/.sawmillrc' to '~/.sawfishrc'\n"
            if os.path.exists (os.path.join (get_home_dir (), '.sawmill')) and not os.path.exists (os.path.join (get_home_dir (), '.sawfish')):
                text = text + "     Copy '~/.sawmill/' to '~/.sawfish/'\n"
        self.get_page ().set_text (text)

    def on_finish (self):
        global sawmill_needed
        global apply_wm
        global apply_panel
        global apply_nautilus
        global apply_theme

        if debug_mode:
            print "exiting with no action, we're in debug mode"
            sys.exit ()
        
        # before sed'ing the config files in fix_sawmill ()
        gnome.config.sync ()

        if sawmill_needed:
            fix_sawmill ()

        if apply_wm == TRUE:
            conf = load_e_config ()
            save_sawfish_config (conf)
            set_sawfish_default ()

        if apply_panel == TRUE:
            os.system ("/bin/rm -rf ~/.gnome/panel*")

        if apply_nautilus == TRUE:
            switch_to_nautilus ()

        if apply_theme == TRUE:
            switch_to_raleigh ()

        mark_already_run ()

        sys.exit ()

def on_druid_cancel (dummy):
    sys.exit ()

gladefile = "/usr/share/gnome-upgrade/glade/gnome-upgrade.glade"
if debug_mode:
    print "Opening glade file %s" % gladefile

xml = libglade.GladeXML(gladefile, "toplevel")

page_list.append (TitlePage ())

if nautilus_needed:
    page_list.append (NautilusPage ())

if wm_needed:
    page_list.append (WmPage ())

if panel_needed:
    page_list.append (PanelPage ())

if theme_switch_forced:
    page_list.append (ThemePage ())
elif check_theme_switch:
    page_list.append (ThemeSwitchPage ())

if debug_mode:
    print page_list

# do we need to run the druid at all?
if (len (page_list) == 1 and sawmill_needed == FALSE and apply_theme == FALSE):
    if debug_mode:
        print "No pages to display"
    mark_already_run ()
    sys.exit ()

page_list.append (FinishPage ())

if len (page_list) != next_page_num:
    print "Broken gnome-upgrade.py!"

def main ():
    xml.signal_autoconnect (
        { "on_druid_cancel" : on_druid_cancel })
    window = xml.get_widget ("toplevel")

    window.show_all ()
    gtk.mainloop ()

    print "main quit"
    
if __name__ == "__main__":
    signal.signal (signal.SIGINT, signal.SIG_DFL)
    main ()
    print "all done"
